# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-30 01:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('esi', '0003_hide_tokens_from_admin_site'),
    ]

    operations = [
        migrations.AlterField(
            model_name='token',
            name='access_token',
            field=models.CharField(editable=False, help_text='The access token granted by SSO.', max_length=254),
        ),
    ]
