# Generated by Django 2.0.4 on 2018-04-30 00:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('esi', '0004_remove_unique_access_token'),
    ]

    operations = [
        migrations.AlterField(
            model_name='token',
            name='access_token',
            field=models.TextField(editable=False, help_text='The access token granted by SSO.'),
        ),
        migrations.AlterField(
            model_name='token',
            name='refresh_token',
            field=models.TextField(blank=True, default='', editable=False, help_text='A re-usable token to generate new access tokens upon expiry.'),
        ),
    ]
